unit modfastgraph;
(*
@type: unit
@name: Unit to handle screen graphics, accelerated bitmap modes
@author: Tomasz Biela (Tebe/Madteam)

@description:
<http://www.freepascal.org/docs-html/rtl/graph/index-5.html>

Marcin Żukowski (Eru/TQA): fLine
*)

interface

uses	atari, graph_constants, global_vars; // , global_vars;
	// tablice pomocniczeefunkcji fline
	// TODO 2023 wynaleźć te konkretne dziury na adresy
	var g_modgraph_color_bits: array [0..$3ff] of byte absolute MODGRAPH_LINE_COLOR_BITS_ADDR;
	var g_modgraph_lineLo: array [0..255] of byte absolute MODGRAPH_LINE_COLOR_BITS_ADDR + $400;
	var g_modgraph_lineHi: array [0..255] of byte absolute MODGRAPH_LINE_COLOR_BITS_ADDR + $400 + 256;
	var g_modgraph_div4: array [0..255] of byte absolute MODGRAPH_LINE_COLOR_BITS_ADDR + $400 + 256 + 256;

{$i memory_addr.inc}

var
	GraphResult: byte;

//	GetColor: byte;

	VideoRAM: pointer; // TODO2023 czy to faktycznie potrzebne?

	procedure draw_chars(col,py : byte; s : tstring);
	procedure draw_char(px,py,digit : byte); keep;
	procedure draw_hud; inline;
	// procedure InitGraph7; // (mode: byte); overload;
	procedure simplified_init_dlist_mode_7;
	procedure simplified_init_graph7;

	procedure SetBkColor(color: byte); assembler;
	//procedure SetClipRect(x0,y0,x1,y1: smallint); overload;
	//procedure SetClipRect(Rect: TRect); overload;
	procedure SetColor(color: byte); assembler;
//	procedure CloseGraph; assembler;

	procedure clip_line(xs,ys,xe,ye : smallint); // implementacja w graph_clipline.inc
	procedure draw_on_font(col : byte; y : byte; d : byte);
	procedure fLine(x0,y0,x1,y1: byte); assembler; keep;
	procedure SetActiveBuffer(a: word); assembler; overload;
	//procedure SetActiveBuffer(var a: TDisplayBuffer); overload;
	procedure mod_PutPixel(px,py: smallint); assembler; register; keep;

implementation

var

	CurrentX, CurrentY: smallint;

	///{$i graph_clipline.inc}
	/// jarek clipline start ============================
	
	const GR_CLIP_CODE_UP = 1;
	const GR_CLIP_CODE_DN = 2;
	const GR_CLIP_CODE_LE = 4;
	const GR_CLIP_CODE_RT = 8;

	// mutuje code
	procedure calc_code_(x, y : smallint; var code : Byte);
	begin
		code := 0;
		if (y < GR_CLIP_YU) 	 then code := code OR GR_CLIP_CODE_UP
		else if (Y > GR_CLIP_YD) then code := code OR GR_CLIP_CODE_DN;
		
		if (x < GR_CLIP_XL) 	 then code := code OR GR_CLIP_CODE_LE
		else if (x > GR_CLIP_XR) then code := code OR GR_CLIP_CODE_RT;
	end;
	
	procedure operationS(ae : smallint; bC : byte; bs, be : smallint; var as : smallint);
	begin
		as := as + smallint(ae - as) * smallint(bC - bs) div smallint(be - bs);
	end;
	
	procedure operationE(as : smallint; bC : byte; be, bs : smallint; var ae : smallint);
	begin
		ae := ae + smallint(ae - as) * smallint(bC - be) div smallint(be - bs);
	end;
	
	procedure clip_line(xs,ys,xe,ye : smallint);
	var
		SCode : Byte;
		ECode : Byte;
		swap : smallint;
	begin
		//if xs < 10 then xs := 10;
		//if xe < 10 then xe := 10;
		//if ys < 130 then ys := 130;
		//if ye < 130 then ye := 10;
	
		if (xs > xe) then begin
			swap:=xe; 
			xe:=xs; 
			xs:=swap; 
			swap:=ye;
			ye:=ys;
			ys:=swap;
		end;		
		calc_code_(xs, ys, SCode);
		calc_code_(xe, ye, ECode);
		// Wxriteln('SCode ', SCode, ' ECode ', ECode);
		//readkey;
		
		if ((ECode AND SCode) <> 0) then begin
			exit;
		end;
		
		if ((ECode OR SCode) = 0) then begin
			{$ifndef DISABLE_LINES}
					// fline(xs,ys,xe,ye);
					g_line_x0 := xs;	//0; // xs;
					g_line_y0 := ys;	//0; // ys;
					g_line_x1 := xe;	//159; // xe;
					g_line_y1 := ye;	//63; // ye;
					fline(g_line_x0,g_line_y0,g_line_x1,g_line_y1);
			{$endif}
			{$ifdef DEBUG_PLOT_VERTEXES}
				mod_putpixel(xs,ys);
				mod_putpixel(xe,ye);
			{$endif}
			exit;
		end;
		
		while (SCode <> 0) do begin
			if (SCode AND GR_CLIP_CODE_LE <> 0) then begin
				// ys := ys + (ye - ys) * (GR_CLIP_XL - xs) div (xe - xs);
				{ys := }operationS(ye, GR_CLIP_XL, xs, xe, ys);
				xs := GR_CLIP_XL;
			end else if (SCode AND GR_CLIP_CODE_RT <> 0) then begin
				//ys := ys + (ye - ys) * (GR_CLIP_XR - xs) div (xe - xs);
				{ys := }operationS(ye, GR_CLIP_XR, xs, xe, ys);
				xs := GR_CLIP_XR;
			end else if (SCode AND GR_CLIP_CODE_UP <> 0) then begin
				// xs := xs + (xe - xs) * (GR_CLIP_YU - ys) div (ye - ys);
				{xs := }operationS(xe, GR_CLIP_YU, ys, ye, xs);
				ys := GR_CLIP_YU;
			end else if (SCode AND GR_CLIP_CODE_DN <> 0) then begin
				// xs := xs + (xe - xs) * (GR_CLIP_YD - ys) div (ye -ys);
				{xs := }operationS(xe, GR_CLIP_YD, ys, ye, xs);
				ys := GR_CLIP_YD;
			end;
			calc_code_(xs,ys,SCode);
			if ((ECode AND SCode) <> 0) then begin
				exit;
			end;
			
			if ((ECode OR SCode) = 0) then begin
				{$ifndef DISABLE_LINES}
					// fline(xs,ys,xe,ye);
					g_line_x0 := xs;	//0; // xs;
					g_line_y0 := ys;	//0; // ys;
					g_line_x1 := xe;	//159; // xe;
					g_line_y1 := ye;	//63; // ye;
					fline(g_line_x0,g_line_y0,g_line_x1,g_line_y1);
				{$endif}
				{$ifdef DEBUG_PLOT_VERTEXES}
					mod_putpixel(xs,ys);
					mod_putpixel(xe,ye);
				{$endif}
				exit;
			end;
		end;
		
		while (ECode <> 0) do begin
			if (ECode AND GR_CLIP_CODE_RT <> 0) then begin
				// ye := ye + (ye - ys) * (GR_CLIP_XR - xe) div (xe - xs);
				{ye := }operationE(ys, GR_CLIP_XR, xe, xs, ye);
				xe := GR_CLIP_XR;
			end else if (ECode AND GR_CLIP_CODE_UP <> 0) then begin
				// xe := xe + (xe - xs) * (GR_CLIP_YU - ye) div (ye - ys);
				{xe := }operationE(xs, GR_CLIP_YU, ye, ys, xe);
				ye := GR_CLIP_YU;
			end else if (ECode AND GR_CLIP_CODE_DN <> 0) then begin
				// xe := xe + (xe - xs) * (GR_CLIP_YD - ye) div (ye - ys);
				{xe := }operationE(xs, GR_CLIP_YD, ye, ys, xe);
				ye := GR_CLIP_YD;
			end;
			calc_code_(xe,ye, ECode);
			if (ECode AND SCODE <> 0) then begin
				exit;
			end;
			
			if ((ECode OR SCode) = 0) then begin
	//			{$ifndef DISABLE_LINES}
					//fline(xs,ys,xe,ye);
					g_line_x0 := xs;	// 0; // xs;
					g_line_y0 := ys;	// 0; // ys;
					g_line_x1 := xe;	// 159; // xe;
					g_line_y1 := ye;	// 63; // ye;
					//if (g_line_x0 >= 160) then halt;
					//if (g_line_x1 >= 160) then halt;
					
					//if (g_line_y0 >= 64) then halt;
					//if (g_line_y1 >= 64) then halt;

					fline(g_line_x0,g_line_y0,g_line_x1,g_line_y1);
		//		{$endif}
			//	{$ifdef DEBUG_PLOT_VERTEXES}
				//	mod_putpixel(xs,ys);
//					mod_putpixel(xe,ye);
	//			{$endif}
				exit;
			end;
		end;
		
		// wxriteln('xs ', xs, ' ys ', ys, ' xe ', xe, ' ye ', ye);
		// readkey;
	end;
	/// jarek clipline end ============================
	
	
	// JAREK
	procedure draw_hud; inline;
	begin
		asm
			txa:pha
			lda graph_pageflip.g_pg
			cmp #1
			bne page_one
			lda #$16  // adres startowy strony 0
			jmp fill
page_one:
			lda #$C0 // adres video strony 1

fill:
			sta :bp2+1 // hi addres
			// offset od którego zaczynamy rysowanie - prawy górny róg
			lda #208 // #5 * #40 + #10
			sta :bp2 // w bp2 adres bazowy
			
			lda #$ff	// kolor szary na 4 pikselach
			ldy #22		// szerokość poziomej kreski
			; rysowanie poziomej kreski od prawej do lewej strony
loop_horizontal:
			sta (:bp2),y
			dey
			bne loop_horizontal
			
			ldx #58
			
loop_vertical:
			ldy #0
			lda :bp2
			clc
			adc #40 // :bp2+#40
			sta :bp2 // chcemy tak przejć do adresu nowego wiersza w tej samej kolumnie
			lda :bp2+1
			adc #0	// tylko carry dodaje
			sta :bp2+1
			
			lda #%01111101
			sta (:bp2),y
			ldy #23
			sta (:bp2),y
			
			
			dex
			bne loop_vertical
end_hud:
			pla:tax
		end;
	end;

procedure SetBkColor(color: byte); assembler;
(*
@description:
Sets the background color to Color
*)
asm
	sta colbaks
end;

// TODO 2023 rozdzielic na put pixel color i fline color
procedure SetColor(color: byte); assembler;
(*
@description:
Sets the foreground color to Color
*)
asm
;;;	sta GetColor

;;;	jmp gr15
;;; mode	equ *-2

gr15	lda color
	and #3
	tay

	jmp toend

;;;;gr9	txa:pha
;;;;
;;;;	lda color
;;;;	and #$0f
;;;;	sta c9+1
;;;;	:4 asl @
;;;;	sta c9
;;;;
;;;;	ldy #0
;;;;lp	tya
;;;;	and #1
;;;;	tax
;;;;	lda c9_,x
;;;;	sta adr.g_modgraph_color_bits,y
;;;;	lda c9,x
;;;;	sta adr.g_modgraph_color_bits+$100,y
;;;;	iny
;;;;	bne lp
;;;;
;;;;	pla:tax
;;;;
;;;;	ldy color
;;;;	beq toend
;;;;
;;;;	ldy #1
;;;;	jmp toend

;;;; c9	dta 0,0
;;;; c9_	dta $0f, $f0

colorHi	dta h(adr.g_modgraph_color_bits, adr.g_modgraph_color_bits+$100, adr.g_modgraph_color_bits+$200, adr.g_modgraph_color_bits+$300)

toend
	// .ifdef fLine
	lda colorHi,y
	sta fLine.urr_color+2
	sta fLine.uur_color+2
	sta fLine.drr_color+2
	sta fLine.ddr_color+2

	lda #$1d		; ora *,x
	cpy #0
	sne
	lda #$3d		; and *,x

	sta fLine.urr_color
	sta fLine.uur_color
	sta fLine.drr_color
	sta fLine.ddr_color
	// .endif

	//.ifdef PutPixel
	mva colorHi,y mod_PutPixel.acol+2

	lda #$1d		; ora *,x
	cpy #0
	sne
	lda #$3d		; and *,x

	sta mod_PutPixel.acol
	//.endif
end;



	procedure draw_chars(col,py : byte; s : tstring);
	var i : byte;
		amt : byte;
	begin
		amt := byte(s[0]);
		for i:=1 to amt do begin
			if (byte(s[i]) <> 32) then begin // jeśli nie spacja
				draw_char(col + i - 1, py,byte(s[i])-48);
			end 
		end;
	end;
	
	// JAREK 49 to 0
	procedure draw_char(px,py,digit : byte); keep; // z jakichś względów wymagane
	begin

		asm
		txa:pha
		
		// adres względem y
		ldy py
		lda adr.g_modgraph_lineLo,y
		// to zmieniane przez setactivepage
		add lfb:#0
		sta :bp2

		lda adr.g_modgraph_lineHi,y
		adc hfb:#0
		sta :bp2+1
		
		// dodanie X
		//ldx px
		//ldy adr.g_modgraph_div4,x
		ldy px // wersja odd razu dodania kolumny - wyrownanie kolumnowe
		
		lda (:bp2),y
		//ora 3
		ldx digit
		lda GRAPH_CONSTANTS.digits_address,x // na początku są offsety znakow
		tax
		lda (:bp2),y
		ora GRAPH_CONSTANTS.digits_address,x+0
		sta (:bp2),y
			
			// przejscie do kolejnej linii
			tya
			adc #40
			tay
			
		//inx
		lda (:bp2),y
		ora GRAPH_CONSTANTS.digits_address,x+1
		sta (:bp2),y
		
			// przejscie do kolejnej linii
			tya
			adc #40
			tay

		// lda #$88;
		//inx
		lda (:bp2),y
		ora GRAPH_CONSTANTS.digits_address,x+2
		
		sta (:bp2),y
		
			// przejscie do kolejnej linii
			tya
			adc #40
			tay
			
		//inx
		lda (:bp2),y
		ora GRAPH_CONSTANTS.digits_address,x+3
		sta (:bp2),y

			// przejscie do kolejnej linii
			tya
			adc #40
			tay

		//inx
		lda (:bp2),y
		ora GRAPH_CONSTANTS.digits_address,x+4;
		sta (:bp2),y
			
		pla:tax
		end;
	end;
	
procedure mod_PutPixel(px,py: smallint); assembler; register; keep; // keep bo inaczej nie kompiluje jak nie uzyjemy - natomiast daloby sie przywrocic ifdefy
asm
	stx @sp

	lda py+1
	bmi stop
	cmp #0 ;;hsh:#0	;MAIN.SYSTEM.ScreenHeight+1
	bne sk0
	lda py
	cmp #64 ;; JAREK hardcode wymiaru lsh:#0	;MAIN.SYSTEM.ScreenHeight
sk0
	bcs stop

	lda px+1
	bmi stop
	cmp #0 ; //hsw:#0	;MAIN.SYSTEM.ScreenWidth+1
	bne sk1
	lda px
	cmp #160; JAREK hardcode wymiaru //lsw:#0	;MAIN.SYSTEM.ScreenWidth
sk1
	bcs stop

	ldy py
	lda adr.g_modgraph_lineLo,y
	add lfb:#0
	sta :bp2

	lda adr.g_modgraph_lineHi,y
	adc hfb:#0
	sta :bp2+1

	;;jmp gr8
;;mode	equ *-2

; ----------------------------

gr15	ldx px
	ldy adr.g_modgraph_div4,x

plot	lda (:bp2),y
acol	and adr.g_modgraph_color_bits,x
	sta (:bp2),y

stop	ldx #0
@sp	equ *-1

end;


function GetPixel(x,y: smallint): byte; assembler;
(*
@description:
Return color of pixel
*)
asm
{	txa:pha

	lda y+1
	bmi error
	cmp MAIN.SYSTEM.ScreenHeight+1
	bne sk0
	lda y
	cmp MAIN.SYSTEM.ScreenHeight
sk0
	bcs error

	lda x+1
	bmi error
	cmp MAIN.SYSTEM.ScreenWidth+1
	bne sk1
	lda x
	cmp MAIN.SYSTEM.ScreenWidth
sk1
	bcs error

	ldy y
	lda adr.g_modgraph_lineLo,y
	add lfb:#0
	sta :bp2

	lda adr.g_modgraph_lineHi,y
	adc hfb:#0
	sta :bp2+1

	jmp gr15
mode	equ *-2


error	lda #0
	jmp stop


gr8	.local
	lda x
	tax

	lsr x+1
	ror @

	:2 lsr @
	tay

	txa
	and #7
	tax

	lda (:bp2),y
	and msk,x
	bne c1

	jmp stop

c1	lda #1
	jmp stop

msk	dta $80,$40,$20,$10,$08,$04,$02,$01

	.endl



gr15	.local

	ldx x
	ldy adr.g_modgraph_div4,x

	txa
	and #3
	beq _0

	cmp #1
	beq _1

	cmp #2
	beq _2

_3	lda (:bp2),y
	and #$03
	jmp stop

_0	lda (:bp2),y
	and #$c0
	:6 lsr @
	jmp stop

_1	lda (:bp2),y
	and #$30
	:4 lsr @
	jmp stop

_2	lda (:bp2),y
	and #$0c
	:2 lsr @
	jmp stop

	.endl


gr9	.local

	lda x
	lsr @
	tay

	lda x
	and #1
	bne _1

_0	lda (:bp2),y
	:4 lsr @
	jmp stop

_1	lda (:bp2),y
	and #$0f
	jmp stop

	.endl

stop	sta Result

	pla:tax
};
end;

// TODO 2023 oczywiście na asembler
procedure draw_on_font(col : byte; y : byte; d : byte);
var	yy : byte;
begin

	g_a := 59 + col;
	g_a := g_a SHL 3;
	g_b := y SHR 3; // ktory wiersz znakowy
	g_b := g_b * 80;
	
	yy := y AND %111;
	poke(FONT_ADDR + g_a + g_b + yy, d);
	//show_message(7,'OF');
	//CRT_write(FONT_ADDR + g_dividend16 + g_divisor8 + yy);
end;


procedure fLine(x0,y0,x1,y1: byte); assembler; keep; // keep bo inaczej nie kompiluje jak nie uzyjemy - natomiast daloby sie przywrocic ifdefy
(*
@description:

DRAWTO in Graphics 8, 9, 15
A quick hack by eru
*)

asm
dx	= ztmp
dy	= ztmp+1
tmp	= ztmp+2
todo	= ztmp+3

PIXEL	.MACRO
	ldy adr.g_modgraph_div4,x
	lda (:bp2),y
	.def :1 = *
	ora adr.g_modgraph_color_bits,x
	sta (:bp2),y
	.ENDM

PREPARE	.MACRO
	sta todo
	inc todo
	lsr @
	sta tmp
	.ENDM

; ==========================================================================
drawto
	txa:pha

; check if going right (x1 <= x0)
	ldx x1
	cpx x0
	bcs right
; going left, swap points
	lda x0
	stx x0
	sta x1
	tax
	ldy y0
	lda y1
	sty y1
	sta y0
right
; compute X delta
	txa
	sec
	sbc x0
	sta dx
; set initial line address
	ldy y0
	lda adr.g_modgraph_lineLo,y
	add lfb:#0
	sta :bp2
	lda adr.g_modgraph_lineHi,y
	adc hfb:#0
	sta :bp2+1

; remember x0 in X
	ldx x0
; check if going up or down
	cpy y1
	jcc down

; ----------------------------- UP ----------------------------------
up
; compute Y delta
	lda y0
	sec
	sbc y1
	sta dy
; check if UP UP RIGHT or UP RIGHT RIGHT
	cmp dx
	bcs up_up_right
up_right_right
	lda dx
	PREPARE urr_color
urr_loop
	PIXEL urr_color
	inx			; go 1 pixel right
	lda tmp
	sec
	sbc dy
	sta tmp
	bcs urr_skip
	adc dx
	sta tmp
	lda :bp2		; go 1 line up
	sec
	sbc #0
w0	equ *-1
	sta :bp2
	bcs *+4
	dec :bp2+1
urr_skip
	dec todo
	bne urr_loop

stop_	jmp stop

up_up_right
	lda dy
	PREPARE uur_color
uur_loop
	PIXEL uur_color
	lda :bp2		; go 1 line up
	sec
	sbc #0
w1	equ *-1
	sta :bp2
	bcs *+4
	dec :bp2+1
	lda tmp
	sec
	sbc dx
	bcs uur_skip
	adc dy
	inx			; go 1 pixel right
uur_skip
	sta tmp
	dec todo
	bne uur_loop

	jmp stop

; ----------------------------- DOWN ----------------------------------
down
; compute Y delta
	lda y1
	sec
	sbc y0
	sta dy
; check if DOWN DOWN RIGHT or DOWN RIGHT RIGHT
	cmp dx
	bcs down_down_right
down_right_right
	lda dx
	PREPARE drr_color
drr_loop
	PIXEL drr_color
	inx			; go 1 pixel right
	lda tmp
	sec
	sbc dy
	sta tmp
	bcs drr_skip
	adc dx
	sta tmp
	lda :bp2		; go 1 line down
	clc
	adc #0
w2	equ *-1
	sta :bp2
	bcc *+4
	inc :bp2+1
drr_skip
	dec todo
	bne drr_loop

	jmp stop

down_down_right
	lda dy
	PREPARE ddr_color
ddr_loop
	PIXEL ddr_color
	lda :bp2		; go 1 line down
	clc
	adc #0
w3	equ *-1
	sta :bp2
	bcc *+4
	inc :bp2+1
	lda tmp
	sec
	sbc dx
	bcs ddr_skip
	adc dy
	inx			; go 1 pixel right
ddr_skip
	sta tmp
	dec todo
	bne ddr_loop

stop	pla:tax
end;


{function Scanline(y: smallint): PByte;
var i: byte;
begin

 i:=y;

 if y < 0 then i:=0 else
  if y >= ScreenHeight then i:=ScreenHeight-1;

 Result:=pointer(VideoRam + g_modgraph_lineLo[i] + g_modgraph_lineHi[i] shl 8);

end;
}


procedure SetActiveBuffer(a: word); assembler; overload;
(*
@description:

*)
asm
	lda a
	ldy a+1

	sta VideoRam
	sty VideoRam+1

	// .ifdef PutPixel
	 sta mod_PutPixel.lfb
	 sty mod_PutPixel.hfb
	//eif
	
	 sta draw_char.lfb
	 sty draw_char.hfb


	.ifdef GetPixel
	 sta GetPixel.lfb
	 sty GetPixel.hfb
	eif

	.ifdef HLine
	 sta HLine.lfb
	 sty HLine.hfb
	eif

	//.ifdef fLine
	 sta fLine.lfb
	 sty fLine.hfb
	//eif

	//.ifdef PutPixel
	 ;;lda WIN_RIGHT
	 ;;add #1
	 ;;sta PutPixel.lsw
	 ;;lda WIN_RIGHT+1
	 ;;adc #0
	 ;;sta PutPixel.hsw

	 ;;lda WIN_BOTTOM
	 ;;add #1
	 ;;sta PutPixel.lsh
	 ;;lda WIN_BOTTOM+1
	 ;;adc #0
	 ;;sta PutPixel.hsh
	//eif

end;


{procedure SetActiveBuffer(var a: TDisplayBuffer); overload;
begin

 SetActiveBuffer(a.bp);

end;}

procedure simplified_init_dlist_mode_7;
begin;
	dpoke(dlist_address + 4, word(VIDEO_PAGE_0_ADDR)); // video ram
	dpoke(dlist_address + 70, txt_lines_address); // text ram
	dpoke(dlist_address + 80, dlist_address); // ret
	// SetActiveBuffer(word(($b560));
	ScreenHeight := 64;
	// aktywacja display listy
	SDLSTL := word(dlist_address);
end;


procedure simplified_init_graph7;
var width: byte;
var mode : byte;
begin
	//GraphM
	mode := 7 + 16; // mode;

	width := 7; // mode and $0f;

	asm
		txa:pha		; najpierw X->A, potem push A
		.ifdef MAIN.@DEFINES.ROMOFF
		inc portb
		.endif
		dindex	equ $57
		lda #7
		sta dindex			; dla spokoju wpisze nr trybu
	;end;
	;simplified_init_dlist_mode_7;
	;asm

	ldx #$0 ; JAREK dodane
	lda #$28 ; JAREK dodane

	sta MAIN.SYSTEM.ScreenWidth			; moze to niepotrzebne?
	stx MAIN.SYSTEM.ScreenWidth+1

	ldy #$60 ; JAREK
	
	sty MAIN.SYSTEM.ScreenHeight
	lda #0
	sta MAIN.SYSTEM.ScreenHeight+1
	lda #$28 ; JAREK dodane
	sta width ; JAREK

		
		.ifdef fLine
	;	lda width
		lda #$28			; JAREK  $28=dec40 tak wyszlo z debuga wygląda na długość wiersza w bajtach (160 pikseli = 4pix per bajt*40 bajtów)
		sta fLine.w0
		sta fLine.w1
		sta fLine.w2
		sta fLine.w3
		.endif
		
	.ifdef MAIN.@DEFINES.ROMOFF
	dec portb
	.endif

; tu jest inicjalizacja tabeli dla fline
; ---	init_tabs

	ldy #0
	sty :bp2
	sty :bp2+1
it1
	lda :bp2+1
	sta adr.g_modgraph_lineHi,y
	lda :bp2
	sta adr.g_modgraph_lineLo,y
	clc
	adc width
	sta :bp2
	scc
	inc :bp2+1

	iny
	bne it1

	mva #$55 _col+1

	ldx #3
	stx _and+1
	txa
l0	sta __oras,x
	asl @
	asl @
	dex
	bpl l0

	lda mode
	and #$0f
	cmp #8
	beq x8
	cmp #6
	beq x8

	jmp it2
x8
	mva #$ff _col+1

	ldx #7
	stx _and+1
	lda #1
l1	sta __oras,x
	asl @
	dex
	bpl l1

it2	tya
_and	and #3
	tax
	lda __oras,x
	eor #$ff
	sta adr.g_modgraph_color_bits+$000,y ; color0
	lda __oras,x
_col	and #$55
	sta adr.g_modgraph_color_bits+$100,y ; color1
	lda __oras,x
	and #$aa
	sta adr.g_modgraph_color_bits+$200,y ; color2
	lda __oras,x
	and #$ff
	sta adr.g_modgraph_color_bits+$300,y ; color3
	tya
	lsr @
_nop	lsr @
	sta adr.g_modgraph_div4,y
	iny
	bne it2

	jmp stop

__oras	dta $c0,$30,$0c,$03
	dta 0,0,0,0

		
stop	pla:tax		; najpierw pull accumulator i potem z accumulator do X
	end;

end;

end.
